const { SLAVE_GENERATION_NUMBER } = require('../entity/slave');
const { exec } = require('child_process');
const { slaves } = require('../entity/slave');

/**
 * Initialise le Swarm Manager qui va creer tous les replicas
 */
function initSwarmManager() {
    console.log("Creating ");
    exec("docker swarm leave --force", (err, stdout, stderr) => {
        if (err) {
            console.error(`error: ${err}`);
        } 
    })
    exec("docker swarm init --advertise-addr 127.0.0.1", (err, stdout, stderr) => {
        if (err) {
            console.error(`error: ${err}`);
        } else {
            console.log("Creating slaves using replicas");
            exec("docker service create --restart-condition='none' --network='host' --name slaves --replicas " + SLAVE_GENERATION_NUMBER + " servuc/hash_extractor ./hash_extractor s ws://localhost:3000/slaves", (err, stdout, stderr) => {
                if (err) {
                    console.error(`error: ${err}`);
                } else {
                    setInterval(checkInactive, 5000);
                }
            });
        }
    });
}

/**
 * Verifier qu'il n'y a pas de flemmard en trop et si oui les retires de ca liste
 */
 function checkInactive() {
    let inactifs = getInactiveSlaves();

    if (inactifs.length < SLAVE_GENERATION_NUMBER)
        scaleSlaves(SLAVE_GENERATION_NUMBER - inactifs.length);
    else {
        for (let i = 0; i < inactifs.length - SLAVE_GENERATION_NUMBER; i++) {
            let slave = inactifs[i];
            console.log("Shutdown slave : " + slave.name);
            slave.ws.send("exit");
            slaves = slaves.filter(s => s !== slave)
        }
    }
}

/**
 * Scale le nombre de slave en fonction du nombre de slave manquant
 * @param {*} nbSlavesManquant 
 */
 function scaleSlaves(nbSlavesManquant) {
    let slavesTot = slaves.length + nbSlavesManquant;
    exec("docker service scale slaves=" + slavesTot, (err, _stdout, _stderr) => {
        if (err) {
            console.error(`error: ${err}`);
        } else {
            console.log("Scale slaves to " + slavesTot);
        }
    });
}



/**
 * Arrêt de recherche des les slaves pour un certain hash
 * @param {*} hash 
 */
 function stopSearch(hash) {
    if (hash in hashInSearch) {
        for (let slave of hashInSearch[hash]) {
            slave.ws.send("stop");
            slave.ws.send("exit");
        }
        delete hashInSearch[hash];
        sendNbSlavesToClient();
    }
}

/**
 * Retourne les slaves inactifs
 * @returns {*[]}
 */
function getInactiveSlaves() {
    return slaves.filter(s => !s.active);
}

module.exports = { initSwarmManager, checkInactive, scaleSlaves, stopSearch, getInactiveSlaves }